using Gtk;
using GLib;

namespace Boilerplate {

    public class Application : Gtk.Application {

        const OptionEntry[] options = {
            { "debug", 'd', 0, OptionArg.NONE, null, "Run in debug mode and show extra output", null },
            { "version", 'v', 0, OptionArg.NONE, null, "Return the current application version", null },
            { "disable-csd", 'c', 0, OptionArg.INT, null, "Disable Client Side Decorations (CSD)", null },
            { null }
        }
        
        const GLib.ActionEntry[] actions = {
            {"preferences", on_preferences},
            {"shortcuts", on_shortcuts},
            {"help", on_help},
            {"about", on_about},
            {"quit", on_quit},
        };

        public static Window window;
        public static GLib.Settings settings;
        
        public
        Application () {
            GLib.Object (application_id: "io.github.julianrichen.boilerplate",
                         flags: ApplicationFlags.HANDLES_COMMAND_LINE); // or? Gio.ApplicationFlags.HANDLES_COMMAND_LINE


            Gtk.Window.set_default_icon_name ("io.github.julianrichen.boilerplate"");

            // this. ????
            add_main_option_entries (options);
            add_action_entries (actiona, this);

            this.settings = new GLib.Settings ("io.github.julianrichen.boilerplate");
        }

        protected override void 
        startup () {
            base.startup ();
            this.menu ();
        }
        
        protected override void
        activate () {
            base.activate();

            // Ensure window
            if (window == null) {
                window = new Window (this);
                
                window.destroy.connect (() => {
                    window = null;
                });
            }
            
            // Needed since we have Window(this)?
            // this.add_window(this.window);
            this.window.show_all();
        }

        public override int 
        command_line (ApplicationCommandLine command_line) {
            var args = command_line.get_arguments();
            
            if (args.length > 1) {
                //logger.print(LogMessage.DEBUG, "FeedReader: callback %s".printf(args[1]));
                //callback(args[1]);
            }

            this.activate();

            return 0;
        }
        
        protected override void 
        shutdown () {
            try {
                //logger.print(LogMessage.DEBUG, "Shutdown!");
                base.shutdown();
            } catch(GLib.Error e) {
                //logger.print(LogMessage.ERROR, "FeedReader.shutdown: %s".printf(e.message));
                
            }
        }
        
        private void
        menu () {
            string[,] actions = {
                {"preferences", this.on_preferences},
                {"shortcuts", this.on_shortcuts},
                {"help", this.on_help},
                {"about", this.on_about},
                {"quit", this.on_quit},
            };
            
            for (int i = 0; i < actions.length[0]; i++) {
                var action = new GLib.SimpleAction (actions.length[0][0], null);
                action.activate.connect (() => {
                    actions.length[0][1];
                });
                this.add_action (action);
            }

            var builder = new Gtk.Builder.from_file("resources/ui/app-menu.ui");
            var app_menu = builder.get_object ("app-menu") as GLib.MenuModel;

            set_app_menu (app_menu);
        }

        private void
        build_app_menu () {
            //GLib.SimpleAction action;
            
            var action = new GLib.SimpleAction ("preferences", null);
            action.activate.connect (() => { this.do_preferences (); });
            this.add_action (action);

            action = new GLib.SimpleAction ("shortcuts", null);
            action.activate.connect (() => { this.do_shortcuts (); });
            this.add_action (action);

            action = new GLib.SimpleAction ("help", null);
            action.activate.connect (() => { this.do_help (); });
            this.add_action (action);

            action = new GLib.SimpleAction ("about", null);
            action.activate.connect (() => { this.do_about (); });
            this.add_action (action);

            action = new GLib.SimpleAction ("quit", null);
            action.activate.connect (() => { this.do_quit (); });
            this.add_action (action);

            var builder = new Gtk.Builder.from_file("resources/ui/app-menu.ui");
            var app_menu = builder.get_object ("app-menu") as GLib.MenuModel;

            set_app_menu (app_menu);
        }

        private void
        do_preferences () {
            stdout.printf ("Preferences");
        }

        private void
        do_shortcuts () {
            stdout.printf ("Keyboard Shortcuts");
        }

        private void
        do_help () {
            stdout.printf ("Help");
        }

        private void
        do_about () {
            string[] authors = {
                "Julian Richen <julian@richen.io>"
            };
            string[] artists = {
                "Julian Richen <julian@richen.io>"
            };

            Gtk.show_about_dialog (
                window,
                "artists", artists,
                "authors", authors,
                "program-name", "boilerplate",
                "title", _("Boilerplate"),
                "license-type", Gtk.License.GPL_3_0,
                "logo-icon-name", "io.github.julianrichen.boilerplate",
                "version", "0.1.0",
                "website", "https://github.com/julianrichen/boilerplate-for-vala-gtk3",
                "wrap-license", true
            );
        }

        public void
        do_quit () {
            base.quit ();
        }

    }

}
