using Gtk;
using GLib;

namespace Boilerplate {

    public class Application : Gtk.Application {

        const OptionEntry[] options = {
            { "debug", 'd', 0, OptionArg.NONE, null, "Run in debug mode and show extra output", null },
            { "version", 'v', 0, OptionArg.NONE, null, "Return the current application version", null },
            { "disable-csd", 'c', 0, OptionArg.INT, null, "Disable Client Side Decorations (CSD)", null },
            { null }
        }
        
        const GLib.ActionEntry[] actions = {
            {"preferences", on_preferences},
            {"shortcuts", on_shortcuts},
            {"help", on_help},
            {"about", on_about},
            {"quit", on_quit},
        };

        public static GLib.Settings settings;
        public static Window window;
        public string csd;
        
        public
        Application () {
            GLib.Object (application_id: "io.github.julianrichen.boilerplate",
                         flags: ApplicationFlags.HANDLES_COMMAND_LINE); // or? Gio.ApplicationFlags.HANDLES_COMMAND_LINE

            this.add_main_option_entries (options);

            this.settings = new GLib.Settings ("io.github.julianrichen.boilerplate");
        }

        protected override void 
        startup () {
            base.startup ();
            
            this.add_action_entries (actions, this);

            var builder = new Gtk.Builder ();
            
            try {
                builder.add_from_file ("menu.ui");
            } catch (Error e) {
                //error ("Unable to load file: %s", e.message);
            }

            this.app_menu = builder.get_object ("app-menu") as MenuModel;
        }
        
        protected override int 
        handle_local_options (GLib.VariantDict options) {
            if (options.contains("version")) {
                //print ("%s %s\n", Environment.get_application_name (), Config.VERSION);
                return 0;
            }

            if (options.contains("debug")) {
                // print("Starting debug...");
            }
            
            if (options.contains("disable-csd")) {
                this.csd = false;
            }
            
            return 0;
        }

        protected override void
        activate () {
            base.activate();

            if (this.window == null) {
                this.window = new Window (this);
                
                this.window.destroy.connect (() => {
                    this.window = null;
                });
            }
            
            // Needed since we have Window(this)?
            this.add_window(this.window);
            this.window.show_all();
        }

        public override int 
        command_line (ApplicationCommandLine command_line) {
            // var args = command_line.get_arguments();
            
            this.activate();

            return 0;
        }
        
        protected override void 
        shutdown () {
            try {
                //logger.print(LogMessage.DEBUG, "Shutdown!");
                base.shutdown();
            } catch(GLib.Error e) {
                //logger.print(LogMessage.ERROR, "FeedReader.shutdown: %s".printf(e.message));
            }
        }

        private void
        do_preferences (SimpleAction action, Variant? parameter) {
            stdout.printf ("Preferences");
        }

        private void
        do_shortcuts (SimpleAction action, Variant? parameter) {
            stdout.printf ("Keyboard Shortcuts");
        }

        private void
        do_help (SimpleAction action, Variant? parameter) {
            stdout.printf ("Help");
        }

        private void
        do_about (SimpleAction action, Variant? parameter) {
            string[] authors = {
                "Julian Richen <julian@richen.io>",
                null
            };
            
            string[] artists = {
                "Julian Richen <julian@richen.io>",
                null
            };
            
            string[] documenters = {
                "Julian Richen <julian@richen.io>",
                null
            };

            Gtk.show_about_dialog (
                this.window,
                "program-name", "Boilerplate for Vala & Gtk 3",
                "authors", authors,
                "artists", artists,
                "documenters", documenters,
                "version", "0.1.0",
                "license-type", Gtk.License.GPL_3_0,
                "logo-icon-name", "io.github.julianrichen.boilerplate",
                "website", "https://github.com/julianrichen/boilerplate-for-vala-gtk3",
            );
        }

        public void
        do_quit (SimpleAction action, Variant? parameter) {
            base.quit ();
        }

    }

}